package io.tswf.groovy.bettergroovy.asm.expansion.generator.conflict

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.groovy.bettergroovy.asm.expansion.meta.ExpansionConflictResolveStrategy

@AutoService(IConflictResolveStrategyLogic)
@Slf4j
@CompileStatic
class WarningConflictResolveStrategy implements IConflictResolveStrategyLogic {

    // Загружаем самую приоритетную стратегию для логирования (нужно для окружений, где может быть любой кастомный логгер)
    @Lazy
    IExpansionConflictResolveWarningProvider loggingProvider = ServiceLoader.load(IExpansionConflictResolveWarningProvider)
        .toList()
            .stream()
                .reverseSortedBy { it.priority }
                .findFirst().orElseThrow { new RuntimeException("There is no expansion logging strategies found!") }


    @Override
    void handleConflictResolve(ConflictResolveContext context) {
        loggingProvider.log("Method ${context.conflictingMethod.javaFullName} already exists in ${context.targetClass.javaName}, not adding it from ${context.expansionMethodMeta.expansion.expansionClass.javaName}...")
    }

    @Override
    ExpansionConflictResolveStrategy getEnumValue() {
        return ExpansionConflictResolveStrategy.WARN
    }
}

interface IExpansionConflictResolveWarningProvider {
    void log(String logMessage)
    default int getPriority() { 0 }
}

@Slf4j
@AutoService(IExpansionConflictResolveWarningProvider)
@CompileStatic
class Slf4jExpansionConflictResolveWarningProvider implements IExpansionConflictResolveWarningProvider{

    public static final int PRIORITY = -1

    @Override
    void log(String logMessage) {
        log.warn(logMessage)
    }
}
